@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 PyCOMPSs API - CONSTRAINT
@@ -20,15 +23,16 @@ PyCOMPSs API - CONSTRAINT
     This file contains the class constraint, needed for the constraint
     definition through the decorator.
 """
+
 import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.runtime.binding import register_ce
 from pycompss.util.location import i_am_at_master
+from pycompss.util.location import i_am_within_scope
 
-
-logger = logging.getLogger(__name__)
+if __debug__:
+    logger = logging.getLogger(__name__)
 
 
 class constraint(object):
@@ -36,25 +40,46 @@ class constraint(object):
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on task constraint creation.
     """
+
     def __init__(self, *args, **kwargs):
-        # store arguments passed to the decorator
-        self.args = args
-        self.kwargs = kwargs
-        logger.debug("Init @constraint decorator...")
+        """
+        Store arguments passed to the decorator
         # self = itself.
         # args = not used.
         # kwargs = dictionary with the given constraints.
 
+        :param args: Arguments
+        :param kwargs: Keyword arguments
+        """
+
+        self.args = args
+        self.kwargs = kwargs
+        self.scope = i_am_within_scope()
+        if self.scope and __debug__:
+            logger.debug("Init @constraint decorator...")
+
     def __call__(self, func):
+        """
+        Parse and set the constraints within the task core element.
+
+        :param func: Function to decorate
+        :return: Decorated function.
+        """
+
+        if not self.scope:
+            from pycompss.api.dummy.constraint import constraint as dummy_constraint
+            d_c = dummy_constraint(self.args, self.kwargs)
+            return d_c.__call__(func)
 
         if i_am_at_master():
             # master code
+            from pycompss.runtime.binding import register_ce
 
             mod = inspect.getmodule(func)
-            self.module = mod.__name__    # not func.__module__
+            self.module = mod.__name__  # not func.__module__
 
-            if(self.module == '__main__' or
-               self.module == 'pycompss.runtime.launch'):
+            if (self.module == '__main__' or
+                    self.module == 'pycompss.runtime.launch'):
                 # The module where the function is defined was run as __main__,
                 # we need to find out the real module name.
 
@@ -83,15 +108,16 @@ class constraint(object):
 
             # Include the registering info related to @constraint
 
-            # Retrieve the base coreElement established at @task decorator
-            coreElement = func.__to_register__
+            # Retrieve the base core_element established at @task decorator
+            core_element = func.__to_register__
             # Update the core element information with the constraints
-            coreElement.set_implConstraints(self.kwargs)
-            func.__to_register__ = coreElement
+            core_element.set_impl_constraints(self.kwargs)
+            func.__to_register__ = core_element
             # Do the task register if I am the top decorator
             if func.__who_registers__ == __name__:
-                logger.debug("[@CONSTRAINT] I have to do the register of function %s in module %s" % (func.__name__, self.module))
-                register_ce(coreElement)
+                if __debug__:
+                    logger.debug("[@CONSTRAINT] I have to do the register of function %s in module %s" % (func.__name__, self.module))
+                register_ce(core_element)
         else:
             # worker code
             pass
@@ -99,7 +125,8 @@ class constraint(object):
         @wraps(func)
         def constrained_f(*args, **kwargs):
             # This is executed only when called.
-            logger.debug("Executing constrained_f wrapper.")
+            if __debug__:
+                logger.debug("Executing constrained_f wrapper.")
 
             if len(args) > 0:
                 # The 'self' for a method function is passed as args[0]
@@ -121,5 +148,6 @@ class constraint(object):
                     setattr(slf, k, v)
 
             return ret
+
         constrained_f.__doc__ = func.__doc__
         return constrained_f
