@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 PyCOMPSs API - DECAF
@@ -20,15 +23,16 @@ PyCOMPSs API - DECAF
     This file contains the class decaf, needed for the @decaf task
     definition through the decorator.
 """
+
 import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.runtime.binding import register_ce
 from pycompss.util.location import i_am_at_master
+from pycompss.util.location import i_am_within_scope
 
-
-logger = logging.getLogger(__name__)
+if __debug__:
+    logger = logging.getLogger(__name__)
 
 
 class decaf(object):
@@ -36,33 +40,68 @@ class decaf(object):
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on mpi task creation.
     """
+
     def __init__(self, *args, **kwargs):
-        # store arguments passed to the decorator
-        self.args = args
-        self.kwargs = kwargs
-        logger.debug("Init @decaf decorator...")
-
-        # Get the computing nodes: This parameter will have to go down
-        # until execution when invoked.
-        if 'computingNodes' not in self.kwargs:
-            self.kwargs['computingNodes'] = 1
-        else:
-            self.kwargs['computingNodes'] = kwargs['computingNodes']
-        logger.debug("This DECAF task will have " + str(self.kwargs['computingNodes']) + " computing nodes.")
-
+        """
+        Store arguments passed to the decorator
         # self = itself.
         # args = not used.
         # kwargs = dictionary with the given constraints.
 
+        :param args: Arguments
+        :param kwargs: Keyword arguments
+        """
+
+        self.args = args
+        self.kwargs = kwargs
+        self.scope = i_am_within_scope()
+        if self.scope:
+            if __debug__:
+                logger.debug("Init @decaf decorator...")
+
+            # Get the computing nodes: This parameter will have to go down
+            # until execution when invoked.
+            if 'computingNodes' not in self.kwargs:
+                self.kwargs['computingNodes'] = 1
+            else:
+                computing_nodes = kwargs['computingNodes']
+                if isinstance(computing_nodes, int):
+                    self.kwargs['computingNodes'] = kwargs['computingNodes']
+                elif isinstance(computing_nodes, str) and computing_nodes.strip().startswith('$'):
+                    env_var = computing_nodes.strip()[1:]  # Remove $
+                    if env_var.startswith('{'):
+                        env_var = env_var[1:-1]  # remove brackets
+                    self.kwargs['computingNodes'] = int(os.environ[env_var])
+                else:
+                    raise Exception("Wrong Computing Nodes value at DECAF decorator.")
+            if __debug__:
+                logger.debug("This DECAF task will have " + str(self.kwargs['computingNodes']) + " computing nodes.")
+        else:
+            pass
+
     def __call__(self, func):
+        """
+        Parse and set the decaf parameters within the task core element.
+
+        :param func: Function to decorate
+        :return: Decorated function.
+        """
+
+        if not self.scope:
+            # from pycompss.api.dummy.decaf import decaf as dummy_decaf
+            # d_d = dummy_decaf(self.args, self.kwargs)
+            # return d_d.__call__(func)
+            raise Exception("The decaf decorator only works within PyCOMPSs framework.")
 
         if i_am_at_master():
             # master code
-            mod = inspect.getmodule(func)
-            self.module = mod.__name__    # not func.__module__
+            from pycompss.runtime.binding import register_ce
 
-            if(self.module == '__main__' or
-               self.module == 'pycompss.runtime.launch'):
+            mod = inspect.getmodule(func)
+            self.module = mod.__name__  # not func.__module__
+
+            if (self.module == '__main__' or
+                    self.module == 'pycompss.runtime.launch'):
                 # The module where the function is defined was run as __main__,
                 # we need to find out the real module name.
 
@@ -89,38 +128,39 @@ class decaf(object):
                         break
                 self.module = mod_name
 
-            # Include the registering info related to @MPI
+            # Include the registering info related to @decaf
 
-            # Retrieve the base coreElement established at @task decorator
-            coreElement = func.__to_register__
+            # Retrieve the base core_element established at @task decorator
+            core_element = func.__to_register__
             # Update the core element information with the mpi information
-            coreElement.set_implType("DECAF")
+            core_element.set_impl_type("DECAF")
             if 'workingDir' in self.kwargs:
-                workingDir = self.kwargs['workingDir']
+                working_dir = self.kwargs['workingDir']
             else:
-                workingDir = '[unassigned]'   # Empty or '[unassigned]'
+                working_dir = '[unassigned]'  # Empty or '[unassigned]'
             if 'mpiRunner' in self.kwargs:
                 runner = self.kwargs['mpiRunner']
             else:
                 runner = 'mpirun'
-            dfScript = self.kwargs['dfScript']
+            df_script = self.kwargs['dfScript']
             if 'dfExecutor' in self.kwargs:
-                dfExecutor = self.kwargs['dfExecutor']
+                df_executor = self.kwargs['dfExecutor']
             else:
-                dfExecutor = '[unassigned]'   # Empty or '[unassigned]'
+                df_executor = '[unassigned]'  # Empty or '[unassigned]'
             if 'dfLib' in self.kwargs:
-                dfLib = self.kwargs['dfLib']
+                df_lib = self.kwargs['dfLib']
             else:
-                dfLib = '[unassigned]'   # Empty or '[unassigned]'
-            implSignature = 'DECAF.' + dfScript
-            coreElement.set_implSignature(implSignature)
-            implArgs = [dfScript, dfExecutor, dfLib, workingDir, runner]
-            coreElement.set_implTypeArgs(implArgs)
-            func.__to_register__ = coreElement
+                df_lib = '[unassigned]'   # Empty or '[unassigned]'
+            impl_signature = 'DECAF.' + df_script
+            core_element.set_impl_signature(impl_signature)
+            impl_args = [df_script, df_executor, df_lib, working_dir, runner]
+            core_element.set_impl_type_args(impl_args)
+            func.__to_register__ = core_element
             # Do the task register if I am the top decorator
             if func.__who_registers__ == __name__:
-                logger.debug("[@DECAF] I have to do the register of function %s in module %s" % (func.__name__, self.module))
-                register_ce(coreElement)
+                if __debug__:
+                    logger.debug("[@DECAF] I have to do the register of function %s in module %s" % (func.__name__, self.module))
+                register_ce(core_element)
         else:
             # worker code
             pass
@@ -128,7 +168,8 @@ class decaf(object):
         @wraps(func)
         def decaf_f(*args, **kwargs):
             # This is executed only when called.
-            logger.debug("Executing decaf_f wrapper.")
+            if __debug__:
+                logger.debug("Executing decaf_f wrapper.")
 
             # Set the computingNodes variable in kwargs for its usage in @task decorator
             kwargs['computingNodes'] = self.kwargs['computingNodes']
@@ -153,5 +194,6 @@ class decaf(object):
                     setattr(slf, k, v)
 
             return ret
+
         decaf_f.__doc__ = func.__doc__
         return decaf_f
