@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 PyCOMPSs API - Implement (Versioning)
@@ -20,15 +23,16 @@ PyCOMPSs API - Implement (Versioning)
     This file contains the class constraint, needed for the implement
     definition through the decorator.
 """
+
 import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.runtime.binding import register_ce
 from pycompss.util.location import i_am_at_master
+from pycompss.util.location import i_am_within_scope
 
-
-logger = logging.getLogger(__name__)
+if __debug__:
+    logger = logging.getLogger(__name__)
 
 
 class implement(object):
@@ -36,24 +40,47 @@ class implement(object):
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on mpi task creation.
     """
+
     def __init__(self, *args, **kwargs):
-        # store arguments passed to the decorator
-        self.args = args
-        self.kwargs = kwargs
-        logger.debug("Init @implement decorator...")
+        """
+        Store arguments passed to the decorator
         # self = itself.
         # args = not used.
-        # kwargs = dictionary with the given constraints.
+        # kwargs = dictionary with the given implement parameters.
+
+        :param args: Arguments
+        :param kwargs: Keyword arguments
+        """
+
+        self.args = args
+        self.kwargs = kwargs
+        self.scope = i_am_within_scope()
+        if self.scope and __debug__:
+            logger.debug("Init @implement decorator...")
 
     def __call__(self, func):
+        """
+        Parse and set the implementation parameters within the task core element.
+
+        :param func: Function to decorate
+        :return: Decorated function.
+        """
+
+        if not self.scope:
+            # from pycompss.api.dummy.implement import implement as dummy_implement
+            # d_i = dummy_implement(self.args, self.kwargs)
+            # return d_i.__call__(func)
+            raise Exception("The implement decorator only works within PyCOMPSs framework.")
 
         if i_am_at_master():
             # master code
-            mod = inspect.getmodule(func)
-            self.module = mod.__name__    # not func.__module__
+            from pycompss.runtime.binding import register_ce
 
-            if(self.module == '__main__' or
-               self.module == 'pycompss.runtime.launch'):
+            mod = inspect.getmodule(func)
+            self.module = mod.__name__  # not func.__module__
+
+            if (self.module == '__main__' or
+                    self.module == 'pycompss.runtime.launch'):
                 # The module where the function is defined was run as __main__,
                 # we need to find out the real module name.
 
@@ -80,31 +107,32 @@ class implement(object):
                         break
                 self.module = mod_name
 
-            # Include the registering info related to @MPI
+            # Include the registering info related to @implement
 
-            # Retrieve the base coreElement established at @task decorator
-            coreElement = func.__to_register__
+            # Retrieve the base core_element established at @task decorator
+            core_element = func.__to_register__
             # Update the core element information with the mpi information
-            ce_signature = coreElement.get_ce_signature()
-            implSignature = ce_signature
-            coreElement.set_implSignature(implSignature)
+            ce_signature = core_element.get_ce_signature()
+            impl_signature = ce_signature
+            core_element.set_impl_signature(impl_signature)
 
-            anotherClass = self.kwargs['source_class']
-            anotherMethod = self.kwargs['method']
-            ce_signature = anotherClass + '.' + anotherMethod
-            coreElement.set_ce_signature(ce_signature)
+            another_class = self.kwargs['source_class']
+            another_method = self.kwargs['method']
+            ce_signature = another_class + '.' + another_method
+            core_element.set_ce_signature(ce_signature)
 
             # This is not needed since the arguments are already set by the
             # task decorator.
-            # implArgs = [anotherClass, anotherMethod]
-            # coreElement.set_implTypeArgs(implArgs)
+            # implArgs = [another_class, another_method]
+            # core_element.set_implTypeArgs(implArgs)
 
-            coreElement.set_implType("METHOD")
-            func.__to_register__ = coreElement
+            core_element.set_impl_type("METHOD")
+            func.__to_register__ = core_element
             # Do the task register if I am the top decorator
             if func.__who_registers__ == __name__:
-                logger.debug("[@IMPLEMENT] I have to do the register of function %s in module %s" % (func.__name__, self.module))
-                register_ce(coreElement)
+                if __debug__:
+                    logger.debug("[@IMPLEMENT] I have to do the register of function %s in module %s" % (func.__name__, self.module))
+                register_ce(core_element)
         else:
             # worker code
             pass
@@ -112,7 +140,8 @@ class implement(object):
         @wraps(func)
         def implement_f(*args, **kwargs):
             # This is executed only when called.
-            logger.debug("Executing implement_f wrapper.")
+            if __debug__:
+                logger.debug("Executing implement_f wrapper.")
 
             # The 'self' for a method function is passed as args[0]
             slf = args[0]
@@ -132,5 +161,6 @@ class implement(object):
                 setattr(slf, k, v)
 
             return ret
+
         implement_f.__doc__ = func.__doc__
         return implement_f
