@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 PyCOMPSs API - OPENCL
@@ -20,15 +23,16 @@ PyCOMPSs API - OPENCL
     This file contains the class constraint, needed for the opencl task
     definition through the decorator.
 """
+
 import inspect
 import logging
 import os
 from functools import wraps
-from pycompss.runtime.binding import register_ce
 from pycompss.util.location import i_am_at_master
+from pycompss.util.location import i_am_within_scope
 
-
-logger = logging.getLogger(__name__)
+if __debug__:
+    logger = logging.getLogger(__name__)
 
 
 class opencl(object):
@@ -36,24 +40,47 @@ class opencl(object):
     This decorator also preserves the argspec, but includes the __init__ and
     __call__ methods, useful on mpi task creation.
     """
+
     def __init__(self, *args, **kwargs):
-        # store arguments passed to the decorator
-        self.args = args
-        self.kwargs = kwargs
-        logger.debug("Init @opencl decorator...")
+        """
+        Store arguments passed to the decorator
         # self = itself.
         # args = not used.
-        # kwargs = dictionary with the given constraints.
+        # kwargs = dictionary with the given opencl parameters
+
+        :param args: Arguments
+        :param kwargs: Keyword arguments
+        """
+
+        self.args = args
+        self.kwargs = kwargs
+        self.scope = i_am_within_scope()
+        if self.scope and __debug__:
+            logger.debug("Init @opencl decorator...")
 
     def __call__(self, func):
+        """
+        Parse and set the opencl parameters within the task core element.
+
+        :param func: Function to decorate
+        :return: Decorated function.
+        """
+
+        if not self.scope:
+            # from pycompss.api.dummy.opencl import import opencl as dummy_opencl
+            # d_ocl = dummy_opencl(self.args, self.kwargs)
+            # return d_ocl.__call__(func)
+            raise Exception("The opencl decorator only works within PyCOMPSs framework.")
 
         if i_am_at_master():
             # master code
-            mod = inspect.getmodule(func)
-            self.module = mod.__name__    # not func.__module__
+            from pycompss.runtime.binding import register_ce
 
-            if(self.module == '__main__' or
-               self.module == 'pycompss.runtime.launch'):
+            mod = inspect.getmodule(func)
+            self.module = mod.__name__  # not func.__module__
+
+            if (self.module == '__main__' or
+                    self.module == 'pycompss.runtime.launch'):
                 # The module where the function is defined was run as __main__,
                 # we need to find out the real module name.
 
@@ -80,26 +107,27 @@ class opencl(object):
                         break
                 self.module = mod_name
 
-            # Include the registering info related to @OpenCL
+            # Include the registering info related to @opencl
 
-            # Retrieve the base coreElement established at @task decorator
-            coreElement = func.__to_register__
+            # Retrieve the base core_element established at @task decorator
+            core_element = func.__to_register__
             # Update the core element information with the mpi information
-            coreElement.set_implType("OPENCL")
+            core_element.set_impl_type("OPENCL")
             kernel = self.kwargs['kernel']
             if 'workingDir' in self.kwargs:
-                workingDir = self.kwargs['workingDir']
+                working_dir = self.kwargs['workingDir']
             else:
-                workingDir = '[unassigned]'   # Empty or '[unassigned]'
-            implSignature = 'OPENCL.' + kernel
-            coreElement.set_implSignature(implSignature)
-            implArgs = [kernel, workingDir]
-            coreElement.set_implTypeArgs(implArgs)
-            func.__to_register__ = coreElement
+                working_dir = '[unassigned]'   # Empty or '[unassigned]'
+            impl_signature = 'OPENCL.' + kernel
+            core_element.set_impl_signature(impl_signature)
+            impl_args = [kernel, working_dir]
+            core_element.set_impl_type_args(impl_args)
+            func.__to_register__ = core_element
             # Do the task register if I am the top decorator
             if func.__who_registers__ == __name__:
-                logger.debug("[@OPENCL] I have to do the register of function %s in module %s" % (func.__name__, self.module))
-                register_ce(coreElement)
+                if __debug__:
+                    logger.debug("[@OPENCL] I have to do the register of function %s in module %s" % (func.__name__, self.module))
+                register_ce(core_element)
         else:
             # worker code
             pass
@@ -107,7 +135,8 @@ class opencl(object):
         @wraps(func)
         def opencl_f(*args, **kwargs):
             # This is executed only when called.
-            logger.debug("Executing opencl_f wrapper.")
+            if __debug__:
+                logger.debug("Executing opencl_f wrapper.")
 
             if len(args) > 0:
                 # The 'self' for a method function is passed as args[0]
@@ -129,5 +158,6 @@ class opencl(object):
                     setattr(slf, k, v)
 
             return ret
+
         opencl_f.__doc__ = func.__doc__
         return opencl_f
