@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 Utils - Object sizing algorithm
@@ -24,51 +27,62 @@ from __future__ import print_function
 from sys import getsizeof, stderr
 from itertools import chain
 from collections import deque
+
 try:
     from reprlib import repr
 except ImportError:
     pass
 
 
-def dict_handler(d):
+def _dict_handler(d):
+    """
+    Dictionary to dictionary handler converter.
+
+    :param d: Dictionary
+    :return: Dictionary handler
+    """
     return chain.from_iterable(d.items())
 
 
-def total_sizeof(o, handlers={}, verbose=False):
+def total_sizeof(o, handlers=None, verbose=False):
     """
     Returns the approximate memory footprint an object and all of its contents.
-
     Automatically finds the contents of the following builtin containers and
     their subclasses:  tuple, list, deque, dict, set and frozenset.
     To search other containers, add handlers to iterate over their contents:
+        handlers = {SomeContainerClass: iter, OtherContainerClass: OtherContainerClass.get_elements}
 
-        handlers = {SomeContainerClass: iter,
-                    OtherContainerClass: OtherContainerClass.get_elements}
+    :param o: Object to get its size
+    :param handlers: Handlers
+    :param verbose: Verbose mode [ True | False ] (default: False)
+    :return: <Integer> Total size of the object
     """
+
     # dict_handler = lambda d: chain.from_iterable(d.items())
     all_handlers = {tuple: iter,
                     list: iter,
                     deque: iter,
-                    dict: dict_handler,
+                    dict: _dict_handler,
                     set: iter,
                     frozenset: iter,
                     }
-    all_handlers.update(handlers)  # user handlers take precedence
-    seen = set()                   # track which object id's have already been seen
-    default_size = getsizeof(0)    # estimate sizeof object without __sizeof__
+    if handlers is not None:
+        all_handlers.update(handlers)  # user handlers take precedence
+    seen = set()  # track which object id's have already been seen
+    default_size = getsizeof(0)  # estimate sizeof object without __sizeof__
 
-    def sizeof(o):
-        if id(o) in seen:       # do not double count the same object
+    def sizeof(obj):
+        if id(obj) in seen:  # do not double count the same object
             return 0
-        seen.add(id(o))
-        s = getsizeof(o, default_size)
+        seen.add(id(obj))
+        s = getsizeof(obj, default_size)
 
         if verbose:
-            print(s, type(o), repr(o), file=stderr)
+            print(s, type(obj), repr(obj), file=stderr)
 
         for typ, handler in all_handlers.items():
-            if isinstance(o, typ):
-                s += sum(map(sizeof, handler(o)))
+            if isinstance(obj, typ):
+                s += sum(map(sizeof, handler(obj)))
                 break
         return s
 
