@@ -1,5 +1,6 @@
+#!/usr/bin/python
 #
-#  Copyright Barcelona Supercomputing Center (www.bsc.es)
+#  Copyright 2002-2018 Barcelona Supercomputing Center (www.bsc.es)
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,7 +13,9 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
+# 
+
+# -*- coding: utf-8 -*-
 
 """
 PyCOMPSs Worker
@@ -26,19 +29,31 @@ import logging
 import os
 import sys
 import traceback
-from exceptions import ValueError
+import base64
 
 from pycompss.api.parameter import TYPE, JAVA_MAX_INT, JAVA_MIN_INT
+from pycompss.runtime.commons import EMPTY_STRING_KEY
+from pycompss.runtime.commons import STR_ESCAPE
+from pycompss.runtime.commons import IS_PYTHON3
 from pycompss.util.serializer import serialize_to_file
 from pycompss.util.serializer import deserialize_from_file
 from pycompss.util.serializer import deserialize_from_string
 from pycompss.util.serializer import SerializerException
 from pycompss.util.logs import init_logging_worker
+from pycompss.util.persistent_storage import get_by_id
+
+if IS_PYTHON3:
+    long = int
+else:
+    # Exception moved to built-in
+    from exceptions import ValueError
+
+    str_escape = 'string_escape'
 
 SYNC_EVENTS = 8000666
 
 # Should be equal to Tracer.java definitions
-TASK_EVENTS = 8000010
+TASK_EVENTS = 60000100
 
 # Rank 110-119 reserved to events launched from task.py
 PROCESS_CREATION = 100
@@ -60,7 +75,11 @@ if sys.version_info >= (2, 7):
 def compss_worker(persistent_storage):
     """
     Worker main method (invocated from __main__).
+
+    :param persistent_storage: Persistent storage boolean
+    :return: None
     """
+
     logger = logging.getLogger('pycompss.worker.worker')
 
     logger.debug("Starting Worker")
@@ -69,21 +88,22 @@ def compss_worker(persistent_storage):
     path = args[0]
     method_name = args[1]
 
-    numSlaves = int(args[2])
+    num_slaves = int(args[2])
     slaves = []
-    for i in range(2, 2 + numSlaves):
+    for i in range(2, 2 + num_slaves):
         slaves.append(args[i])
-    argPosition = 3 + numSlaves
+    arg_position = 3 + num_slaves
 
-    args = args[argPosition:]
+    args = args[arg_position:]
     cus = args[0]
 
     args = args[1:]
     has_target = args[0]
     return_type = args[1]
-    num_params = int(args[2])
+    num_returns = int(args[2])
+    num_params = int(args[3])
 
-    args = args[3:]
+    args = args[4:]
     pos = 0
 
     values = []
@@ -96,46 +116,34 @@ def compss_worker(persistent_storage):
         pyextrae.event(TASK_EVENTS, PARAMETER_PROCESSING)
 
     if persistent_storage:
-        from storage.api import getByID
-        from storage.api import TaskContext
+        from pycompss.util.persistent_storage import storage_task_context
 
     # Get all parameter values
     logger.debug("Processing parameters:")
     for i in range(0, num_params):
-        pType = int(args[pos])
-        pStream = int(args[pos + 1])
-        pPrefix = args[pos + 2]
-        pValue = args[pos + 3]
+        p_type = int(args[pos])
+        p_stream = int(args[pos + 1])
+        p_prefix = args[pos + 2]
+        p_value = args[pos + 3]
 
         logger.debug("Parameter : " + str(i))
-        logger.debug("\t * Type : " + str(pType))
-        logger.debug("\t * Stream : " + str(pStream))
-        logger.debug("\t * Prefix : " + str(pPrefix))
-        logger.debug("\t * Value: " + str(pValue))
+        logger.debug("\t * Type : " + str(p_type))
+        logger.debug("\t * Stream : " + str(p_stream))
+        logger.debug("\t * Prefix : " + str(p_prefix))
+        logger.debug("\t * Value: " + str(p_value))
 
-        types.append(pType)
-        streams.append(pStream)
-        prefixes.append(pPrefix)
+        types.append(p_type)
+        streams.append(p_stream)
+        prefixes.append(p_prefix)
 
-        if pType == TYPE.FILE:
-            '''
-            # check if it is a persistent object
-            # TODO: I find that it makes no sense to identify PSCOs this way
-            # Why do not we simply check if the object of a subclass of the
-            # storage_object?
-            if 'getID' in dir(pValue) and pValue.getID() is not None:
-                po = getByID(pValue.getID())
-                values.append(po)
-            else:
-                values.append(pValue)
-            '''
-            values.append(pValue)
-        elif pType == TYPE.EXTERNAL_PSCO:
-            po = getByID(pValue)
+        if p_type == TYPE.FILE:
+            values.append(p_value)
+        elif p_type == TYPE.EXTERNAL_PSCO:
+            po = get_by_id(p_value)
             values.append(po)
             pos += 1  # Skip info about direction (R, W)
-        elif pType == TYPE.STRING:
-            num_substrings = int(pValue)
+        elif p_type == TYPE.STRING:
+            num_substrings = int(p_value)
             aux = ''
             first_substring = True
             for j in range(4, num_substrings + 4):
@@ -143,42 +151,53 @@ def compss_worker(persistent_storage):
                     aux += ' '
                 first_substring = False
                 aux += args[pos + j]
-            #######
-            # Check if the string is really an object
-            # Required in order to recover objects passed as parameters.
-            # - Option object_conversion
-            real_value = aux
-            try:
-                # try to recover the real object
-                aux = deserialize_from_string(aux.decode('string_escape'))
-            except (SerializerException, ValueError, EOFError):
-                # was not an object
-                aux = real_value
-            #######
+            # Decode the string received
+            aux = base64.b64decode(aux.encode())
+            if aux.decode() == EMPTY_STRING_KEY:
+                # Then it is an empty string
+                aux = ""
+            else:
+                #######
+                # Check if the string is really an object
+                # Required in order to recover objects passed as parameters.
+                # - Option object_conversion
+                real_value = aux
+                try:
+                    # try to recover the real object
+                    if IS_PYTHON3:
+                        # decode removes double backslash, and encode returns as binary
+                        aux = deserialize_from_string(aux.decode(STR_ESCAPE).encode())
+                    else:
+                        # decode removes double backslash, and str casts the output
+                        aux = deserialize_from_string(str(aux.decode(STR_ESCAPE)))
+                except (SerializerException, ValueError, EOFError):
+                    # was not an object
+                    aux = str(real_value.decode())
+                #######
             values.append(aux)
             logger.debug("\t * Final Value: " + str(aux))
             pos += num_substrings
-        elif pType == TYPE.INT:
-            values.append(int(pValue))
-        elif pType == TYPE.LONG:
-            l = long(pValue)
-            if l > JAVA_MAX_INT or l < JAVA_MIN_INT:
+        elif p_type == TYPE.INT:
+            values.append(int(p_value))
+        elif p_type == TYPE.LONG:
+            my_l = long(p_value)
+            if my_l > JAVA_MAX_INT or my_l < JAVA_MIN_INT:
                 # A Python int was converted to a Java long to prevent overflow
                 # We are sure we will not overflow Python int, otherwise this
                 # would have been passed as a serialized object.
-                l = int(l)
-            values.append(l)
-        elif pType == TYPE.DOUBLE:
-            values.append(float(pValue))
-        elif pType == TYPE.BOOLEAN:
-            if pValue == 'true':
+                my_l = int(my_l)
+            values.append(my_l)
+        elif p_type == TYPE.DOUBLE:
+            values.append(float(p_value))
+        elif p_type == TYPE.BOOLEAN:
+            if p_value == 'true':
                 values.append(True)
             else:
                 values.append(False)
-        # elif (pType == TYPE.OBJECT):
+        # elif (p_type == TYPE.OBJECT):
         #    pass
         else:
-            logger.fatal("Invalid type (%d) for parameter %d" % (pType, i))
+            logger.fatal("Invalid type (%d) for parameter %d" % (p_type, i))
             exit(1)
         pos += 4
 
@@ -216,7 +235,7 @@ def compss_worker(persistent_storage):
             logger.debug("Module successfully loaded (Python version < 2.7")
 
         if persistent_storage:
-            with TaskContext(logger, values, config_file_path=storage_conf):
+            with storage_task_context(logger, values, config_file_path=storage_conf):
                 getattr(module, method_name)(*values, compss_types=types, compss_tracing=tracing)
                 if tracing:
                     pyextrae.eventandcounters(TASK_EVENTS, 0)
@@ -263,7 +282,7 @@ def compss_worker(persistent_storage):
             types.insert(0, TYPE.OBJECT)
 
             if persistent_storage:
-                with TaskContext(logger, values, config_file_path=storage_conf):
+                with storage_task_context(logger, values, config_file_path=storage_conf):
                     getattr(klass, method_name)(*values, compss_types=types, compss_tracing=tracing)
                     if tracing:
                         pyextrae.eventandcounters(TASK_EVENTS, 0)
@@ -279,10 +298,10 @@ def compss_worker(persistent_storage):
             serialize_to_file(obj, file_name)
         else:
             # Class method - class is not included in values (e.g. values = [7])
-            types.insert(0, None)    # class must be first type
+            types.insert(0, None)  # class must be first type
 
             if persistent_storage:
-                with TaskContext(logger, values, config_file_path=storage_conf):
+                with storage_task_context(logger, values, config_file_path=storage_conf):
                     getattr(klass, method_name)(*values, compss_types=types, compss_tracing=tracing)
                     if tracing:
                         pyextrae.eventandcounters(TASK_EVENTS, 0)
@@ -300,8 +319,8 @@ def compss_worker(persistent_storage):
         logger.exception(''.join(line for line in lines))
         exit(1)
 
-if __name__ == "__main__":
 
+if __name__ == "__main__":
     # Emit sync event if tracing is enabled
     tracing = sys.argv[1] == 'true'
     taskId = int(sys.argv[2])
@@ -310,8 +329,8 @@ if __name__ == "__main__":
     method_type = sys.argv[5]
     # class_name = sys.argv[6]
     # method_name = sys.argv[7]
-    # numSlaves = sys.argv[8]
-    # i = 8 + numSlaves
+    # num_slaves = sys.argv[8]
+    # i = 8 + num_slaves
     # slaves = sys.argv[9-i]
     # numCus = sys.argv[i+1]
     # has_target = sys.argv[i+2] == 'true'
@@ -326,6 +345,7 @@ if __name__ == "__main__":
 
     if tracing:
         import pyextrae.multiprocessing as pyextrae
+
         pyextrae.eventandcounters(SYNC_EVENTS, taskId)
         # pyextrae.eventandcounters(TASK_EVENTS, 0)
         pyextrae.eventandcounters(TASK_EVENTS, WORKER_INITIALIZATION)
