@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/common/object_tree.c $
- | @last_commit: $Date: 2015-04-20 16:32:24 +0200 (lun, 20 abr 2015) $
- | @version:     $Revision: 3250 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: object_tree.c 3250 2015-04-20 14:32:24Z harald $";
-
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
@@ -46,7 +39,7 @@ appl_t ApplicationTable;
 void InitializeObjectTable (unsigned num_appl, struct input_t * files,
 	unsigned long nfiles)
 {
-	unsigned int ptask, task, thread, i, j;
+	unsigned int ptask, task, thread, i, j, v;
 	unsigned int ntasks[num_appl], **nthreads = NULL;
 
 	/* First step, collect number of applications, number of tasks per application and
@@ -114,6 +107,7 @@ void InitializeObjectTable (unsigned num_appl, struct input_t * files,
 			task_info->num_binary_objects = 0;
 			task_info->binary_objects = NULL;
 			task_info->thread_dependencies = ThreadDependency_create();
+			task_info->AddressSpace = AddressSpace_create();
 
 			for (thread = 0; thread < nthreads[ptask][task]; thread++)
 			{
@@ -131,9 +125,13 @@ void InitializeObjectTable (unsigned num_appl, struct input_t * files,
 
 				thread_info->dimemas_size = 0;
 				thread_info->virtual_thread = thread+1;
+				thread_info->State_Stack = NULL;
 				thread_info->nStates = 0;
+				thread_info->nStates_Allocated = 0;
 				thread_info->First_Event = TRUE;
 				thread_info->HWCChange_count = 0;
+				for (v = 0; v < MAX_CALLERS; v++)
+					thread_info->AddressSpace_calleraddresses[v] = 0;
 #if USE_HARDWARE_COUNTERS || defined(HETEROGENEOUS_SUPPORT)
 				thread_info->HWCSets = NULL;
 				thread_info->HWCSets_types = NULL;
@@ -176,8 +174,11 @@ static void AddBinaryObjectInto (unsigned ptask, unsigned task,
 	task_t *task_info = GET_TASK_INFO(ptask, task);
 	unsigned found = FALSE, u;
 
-	if (!file_exists(binary))
+	if (!__Extrae_Utils_file_exists(binary))
+	{
+		fprintf (stderr, "mpi2prv: Warning: Couldn't open %s for reading, addresses may not be translated.\n", binary);
 		return;
+	}
 
 	for (u = 0; u < task_info->num_binary_objects && !found; u++)
 		found = strcmp (task_info->binary_objects[u].module, binary) == 0;
@@ -200,6 +201,9 @@ static void AddBinaryObjectInto (unsigned ptask, unsigned task,
 		task_info->binary_objects[last_index].index = last_index+1;
 
 #if defined(HAVE_BFD)
+		task_info->binary_objects[last_index].nDataSymbols = 0;
+		task_info->binary_objects[last_index].dataSymbols = NULL;
+
 		BFDmanager_loadBinary (binary,
 		  &(task_info->binary_objects[last_index].bfdImage),
 		  &(task_info->binary_objects[last_index].bfdSymbols),
@@ -249,6 +253,40 @@ binary_object_t* ObjectTable_GetBinaryObjectAt (unsigned ptask, unsigned task, U
 	return NULL;
 }
 
+int ObjectTable_GetSymbolFromAddress (UINT64 address, unsigned ptask,
+	unsigned task, char **symbol)
+{
+#if !defined(HAVE_BFD)
+	UNREFERENCED_PARAMETER(address);
+	UNREFERENCED_PARAMETER(ptask);
+	UNREFERENCED_PARAMETER(task);
+	UNREFERENCED_PARAMETER(symbol);
+	return FALSE;
+#else
+	unsigned a;
+	task_t *task_info = GET_TASK_INFO(ptask, task);
+
+#if defined(DEBUG)
+	fprintf (stderr, "mpi2prv: DEBUG: ObjectTable_GetSymbolFromAddress (%llx, %u, %u, %p)\n",
+	  address, ptask, task, symbol);
+#endif
+
+	/* For now, emit only data symbols for binary object 0 */
+	for (a = 0; a < task_info->binary_objects[0].nDataSymbols; a++)
+	{
+		data_symbol_t *d = &task_info->binary_objects[0].dataSymbols[a];
+		uint64_t addr_begin = (uint64_t) d->address;
+		uint64_t addr_end = addr_begin + d->size;
+		if (addr_begin <= address && address < addr_end)
+		{
+			*symbol = d->name;
+			return TRUE;
+		}
+	}
+	return FALSE;
+#endif
+}
+
 #if defined(BFD_MANAGER_GENERATE_ADDRESSES)
 void ObjectTable_dumpAddresses (FILE *fd, unsigned eventstart)
 {
