@@ -21,15 +21,8 @@
  *   Barcelona Supercomputing Center - Centro Nacional de Supercomputacion   *
 \*****************************************************************************/
 
-/* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- *\
- | @file: $HeadURL: https://svn.bsc.es/repos/ptools/extrae/trunk/src/merger/parallel/parallel_merge_aux.c $
- | @last_commit: $Date: 2015-02-26 14:38:49 +0100 (jue, 26 feb 2015) $
- | @version:     $Revision: 3125 $
-\* -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
 #include "common.h"
 
-static char UNUSED rcsid[] = "$Id: parallel_merge_aux.c 3125 2015-02-26 13:38:49Z harald $";
-
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -50,13 +43,24 @@ static char UNUSED rcsid[] = "$Id: parallel_merge_aux.c 3125 2015-02-26 13:38:49
 #include "pthread_prv_events.h"
 #include "omp_prv_events.h"
 #include "misc_prv_events.h"
-#include "trt_prv_events.h"
 #include "cuda_prv_events.h"
 #include "addr2info.h"
 #include "options.h"
 
 //#define DEBUG
 
+struct PendingCommunication_t
+{
+	int sender, recver, tag, descriptor, match, match_zone;
+	off_t offset;
+};
+struct PendingComms_t 
+{
+	struct PendingCommunication_t *data;
+	int count, size;
+};
+
+
 static struct ForeignRecv_t **myForeignRecvs;
 static int *myForeignRecvs_count;
 static char **myForeignRecvs_used;
@@ -91,8 +95,8 @@ struct InterCommunicators_t
 static struct IntraCommunicators_t IntraCommunicators;
 static struct InterCommunicators_t InterCommunicators;
 
-struct PendingComms_t PendingComms;
-struct ForeignRecvs_t *ForeignRecvs;
+static struct PendingComms_t PendingComms;
+static struct ForeignRecvs_t *ForeignRecvs;
 
 #define FOREIGN_RECV_RESIZE_STEP ((1024*1024)/sizeof(struct ForeignRecv_t))
 #define INTRA_COMMUNICATORS_RESIZE_STEP ((1024*1024)/sizeof(struct IntraCommunicator_t))
@@ -740,11 +744,6 @@ void ShareTraceInformation (int numtasks, int taskid)
 	fflush (stdout);
 	Share_pthread_Operations ();
 
-	if (0 == taskid)
-		fprintf (stdout, " TRT");
-	fflush (stdout);
-	Share_TRT_Operations ();
-
 	if (0 == taskid)
 		fprintf (stdout, " CUDA");
 	fflush (stdout);
@@ -769,6 +768,11 @@ void ShareTraceInformation (int numtasks, int taskid)
 	Share_Callers_Usage ();
 #endif
 
+	if (0 == taskid)
+		fprintf (stdout, " files");
+	fflush (stdout);
+	Share_File_Names (taskid);
+
 	if (0 == taskid)
 		fprintf (stdout, " >\n");
 	fflush (stdout);
@@ -917,19 +921,22 @@ void Gather_Dimemas_Offsets (int numtasks, int taskid, int count,
 void ShareNodeNames (int numtasks, char ***nodenames)
 {
 	int i, rc;
+	size_t s;
 	char hostname[MPI_MAX_PROCESSOR_NAME];
-	int hostname_length;
 	char *buffer_names = NULL;
 	char **TasksNodes;
 
 	/* Get processor name */
-	rc = MPI_Get_processor_name (hostname, &hostname_length);
-	MPI_CHECK(rc, MPI_Get_processor_name, "Cannot get processor name");
+	if (gethostname (hostname, sizeof(hostname)) == -1)
+	{
+		fprintf (stderr, "Error! Cannot get hostname!\n");
+		exit (-1);
+	}
 
 	/* Change spaces " " into underscores "_" (BGL nodes use to have spaces in their names) */
-	for (i = 0; i < hostname_length; i++)
-		if (' ' == hostname[i])
-			hostname[i] = '_';
+	for (s = 0; s < strlen(hostname); s++)
+		if (' ' == hostname[s])
+			hostname[s] = '_';
 
 	/* Share information among all tasks */
 	buffer_names = (char*) malloc (sizeof(char) * numtasks * MPI_MAX_PROCESSOR_NAME);
