@@ -5,8 +5,14 @@
   ######################
 
   # Load common setup functions --------------------------------------
-  scriptDir=$(dirname $0)
-  source ${scriptDir}/setup.sh
+  if [ -z "${COMPSS_HOME}" ]; then
+    SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
+    export COMPSS_HOME=${SCRIPT_DIR}/../../../../../
+  else
+    SCRIPT_DIR="${COMPSS_HOME}/Runtime/scripts/system/adaptors/nio"
+  fi
+  # shellcheck source=setup.sh
+  source ${SCRIPT_DIR}/setup.sh
 
   # Load parameters --------------------------------------------------
   load_parameters $@
@@ -22,7 +28,10 @@
   # Launch the Worker JVM
   pre_launch  
 
+  reprogram_fpga
+
   if [ "$debug" == "true" ]; then
+      export COMPSS_BINDINGS_DEBUG=1
       export NX_ARGS="--summary"
       echo "Calling NIOWorker"
       echo "Cmd: $cmd ${paramsToCOMPSsWorker}"
